

class InvalidClient(Exception):
    pass


class InvalidRequest(Exception):
    def __init__(self, error : str):
        self.error = error

    def __str__(self) -> str:
        return self.error


class Unauthorized(Exception):
    pass






import aiohttp
from typing import Optional, List
from discord import Permissions
from . import checks


class AccessToken:
    def __init__(self, token_response : dict):
        self._token_response = token_response

    def __str__(self):
        return self.access_token

    @property
    def access_token(self) -> str:
        return self._token_response["access_token"]

    @property
    def token_type(self) -> str:
        return self._token_response["token_type"]

    @property
    def expires_in(self) -> int:
        return self._token_response["expires_in"]

    @property
    def refresh_token(self) -> str:
        return self._token_response["refresh_token"]

    @property
    def scope(self) -> str:
        return self._token_response["scope"]
    
class Oauth2Guild:
    def __init__(self, payload_data : dict):
        self._payload = payload_data

    def __str__(self) -> str:
        return self.name

    @property
    def id(self) -> int:
        return int(self._payload["id"])

    @property
    def name(self) -> str:
        return self._payload["name"]

    @property
    def icon_url(self) -> str:
        icon_hash = self._payload["icon"]
        return "https://cdn.discordapp.com/icons/{}/{}.png".format(self.id, icon_hash)

    @property
    def is_fetched_member_owner(self) -> bool:
        return self._payload["owner"]
    
    @property
    def fetched_member_permissions(self) -> Permissions:
        return Permissions(int(self._payload["permissions"]))

    @property
    def features(self) -> bool:
        return self._payload["features"]

class Oauth2Member:
    def __init__(self, member_payload : dict, access_token : str, session : aiohttp.ClientSession = None):
        self._payload = member_payload
        self.__session__ = session or aiohttp.ClientSession(headers = {'Content-Type': 'application/x-www-form-urlencoded'})
        self.__token__ = access_token
        

    def __str__(self):
        return "{}#{}".format(self.username, self.discriminator)

    
    async def fetch_guilds(self, access_token : Optional[str] = None) -> List[Oauth2Guild]:
        access_token = access_token or self.__token__
        async with self.__session__.get("https://discord.com/api/v8/users/@me/guilds", headers = {"Authorization" : "Bearer {}".format(access_token)}) as response:
            data = await response.json()
            guilds = []
            for guild in data:
                guild_obj = Oauth2Guild(guild)
                guilds.append(guild_obj)

            return guilds



    @property
    def id(self) -> int:
        return int(self._payload["id"])

    @property
    def username(self) -> str:
        return self._payload["username"]

    @property
    def avatar_url(self) -> str:
        avatar_hash = self._payload["avatar"]
        _id = self.id
        return "https://cdn.discordapp.com/avatars/{}/{}.png".format(_id, avatar_hash)

    @property
    def discriminator(self) -> str:
        return self._payload["discriminator"]

    @property
    def public_flags(self) -> int:
        return self._payload["public_flags"]

    @property
    def flags(self) -> int:
        return self._payload["flags"]

    @property
    def locale(self) -> str:
        return self._payload["locale"]

    @property
    def mfa_enabled(self) -> bool:
        return self._payload["mfa_enabled"]

    @property
    def email(self) -> str or None:
        return self._payload["email"]

    @property
    def verified(self) -> bool:
        return self._payload["verified"]


    


    


    


import aiohttp
from . import abc
from .  import checks
from typing import List



#https://discord.com/api/oauth2/authorize?client_id=770301542170361896&redirect_uri=http%3A%2F%2Flocalhost%3A8000&response_type=code&scope=identify%20email

class Oauth2Client:
    def __init__(self, client_id : int, client_secret : str, redirect_uri : str, scopes : List[str]):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.__session__ = aiohttp.ClientSession(headers = {'Content-Type': 'application/x-www-form-urlencoded'}, raise_for_status=True)
        self.oauth2_endpoint = 'https://discord.com/api/v8/oauth2/token'
        self.base_member_endpoint = "https://discord.com/api/v8/users/@me"
        self.scopes = " ".join(scopes)


    async def exchange_code(self, authorization_code : str) -> abc.AccessToken:
        exchange_data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'authorization_code',
            'code': authorization_code,
            'redirect_uri': self.redirect_uri,
            'scope' : self.scopes
        }
        async with self.__session__.post(self.oauth2_endpoint, data = exchange_data) as response:
            data = await response.json()
            token_obj = abc.AccessToken(data)
            return token_obj

    async def fetch_member(self, access_token : str) -> abc.Oauth2Member:
        headers = {"Authorization" : "Bearer {}".format(access_token)}
        async with self.__session__.get(self.base_member_endpoint, headers = headers) as user_response:
            data = await user_response.json()
            member_obj = abc.Oauth2Member(data, access_token)
            return member_obj

    async def refresh_token(self, refresh_token : str) -> abc.AccessToken:
        exchange_data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token
        }
        async with self.__session__.post(self.oauth2_endpoint, data = exchange_data) as response:
            data = await response.json()
            token_obj = abc.AccessToken(data)
            return token_obj

    

    async def close(self):
        return await self.__session__.close()








from setuptools import setup, find_packages
import os







DESCRIPTION = 'An async wrapper for the Discord Oauth2 API'

# Setting up
setup(
    name="discord.web",
    version="0.0.1a",
    author="quiktea",
    author_email="wishymovies@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=open("README.md").read(),
    packages=find_packages(),
    install_requires=['discord.py', 'aiohttp'],
    keywords=['python', 'discord.py', 'dashboard', 'discord', 'api', 'discord api', 'web dashboard', 'website', 'web'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    project_urls={
        "GitHub" : "https://github.com/quiktea/discord-web"
    }
)